// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.
package models

import (
	"errors"
	"leit.com/LAPP_GAAS_GFrame/db"
	"time"
	"xorm.io/core"
)

type MeSetupmatrix struct {
	Plantnr                int       `json:"PlantNr" xorm:"not null pk INT(4)"`
	Worklineid             string    `json:"WorkLineId" xorm:"not null pk NVARCHAR(80)"`
	Fromartid              string    `json:"FromArtId" xorm:"not null pk NVARCHAR(80)"`
	Toartid                string    `json:"ToArtId" xorm:"not null pk NVARCHAR(80)"`
	Descr1                 string    `json:"Descr1" xorm:"not null NVARCHAR(200)"`
	Descr2                 string    `json:"Descr1" xorm:"not null NVARCHAR(200)"`
	Setuptime              float32   `json:"SetupTime" xorm:"not null FLOAT(8)"`
	Timeuomid              string    `json:"TimeUomId" xorm:"not null NVARCHAR(80)"`
	Projectid              string    `json:"ProjectId" xorm:"not null NVARCHAR(80)"`
	Lastmodify             time.Time `json:"LastModify" xorm:"DATETIME(8)"`
	Lastuser               string    `json:"LastUser" xorm:"not null NVARCHAR(40)"`
	Createtime             time.Time `json:"CreateTime" xorm:"DATETIME(8)"`
}

/******数据表名******/
func (t *MeSetupmatrix) TableName() string {
	return "MeSetupMatrix"
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据添加
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) Add() error {
	e := db.Eloquent.Master()
	setuptab := new(MeSetupmatrix)
	/**主键：****/
	affw, err := e.Table(t.TableName()).ID(core.PK{t.Plantnr, t.Worklineid, t.Fromartid, t.Toartid}).Count(setuptab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table(t.TableName()).Insert(t)

	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) Del() error {
	e := db.Eloquent.Master()
	/******/
	_, err := e.ID(core.PK{t.Plantnr, t.Worklineid, t.Fromartid, t.Toartid}).Delete(&MeSetupmatrix{})
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据修改
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) Update() error {
	e := db.Eloquent.Master()
	/******/
	_, err := e.ID(core.PK{t.Plantnr, t.Worklineid, t.Fromartid, t.Toartid}).Update(t)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) SelectOne() (MeSetupmatrix, error) {
	var data MeSetupmatrix

	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Plantnr, t.Worklineid, t.Fromartid, t.Toartid}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) SelectAll() (datalst []MeSetupmatrix, err error) {
	e := db.Eloquent.Master()
	if err = e.Where("PlantNr = ?",t.Plantnr).OrderBy("FromArtId").Find(&datalst); err != nil{
		return
	}
	return
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) SelectProjectAll() (datalst []MeSetupmatrix, err error) {
	e := db.Eloquent.Master()
	if err = e.Where("PlantNr = ? and ProjectId = ?",t.Plantnr, t.Projectid).OrderBy("FromArtId").Find(&datalst); err != nil{
		return
	}
	return
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *MeSetupmatrix) SelectWorklineAll() (datalst []MeSetupmatrix, err error) {
	e := db.Eloquent.Master()
	if err = e.Where("PlantNr = ? and Worklineid = ?",t.Plantnr, t.Worklineid).OrderBy("FromArtId").Find(&datalst); err != nil{
		return
	}
	return
}