// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.
package models

import (
	"leit.com/LAPP_GAAS_GFrame/db"
	"time"
	"errors"
	"xorm.io/core"
)

type Worklinespecialday struct {
	Plantnr    int       `json:"PlantNr" xorm:"not null pk INT(4)"`
	Worklineid string    `json:"WorkLineid" xorm:"not null pk NVARCHAR(80)"`
	Specialday string    `json:"SpecialDay" xorm:"not null pk NVARCHAR(16)"`
	Daymodelnr int       `json:"DayModelNr" xorm:"not null INT(4)"`
	Descr      string    `json:"Descr" xorm:"not null NVARCHAR(200)"`
	Lastmodify time.Time `json:"LastModify" xorm:"DATETIME(8)"`
	Lastuser   string    `json:"LastUser" xorm:"not null NVARCHAR(40)"`
	Createtime time.Time `json:"CreateTime" xorm:"DATETIME(8)"`
}

/******数据表名******/
func (t *Worklinespecialday) TableName() string {
	return "Worklinespecialday"
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据添加
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *Worklinespecialday) Add() error {
	e := db.Eloquent.Master()
	count := new(Worklinespecialday)
	affw, err := e.Table(t.TableName()).ID(core.PK{t.Plantnr, t.Worklineid, t.Specialday}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table(t.TableName()).Insert(t)

	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *Worklinespecialday) Del() bool {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Plantnr, t.Worklineid, t.Specialday}).Delete(&Worklinespecialday{})
	if err != nil {
		return false
	}
	return true
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据修改
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *Worklinespecialday) Update() error {
	e := db.Eloquent.Master()
	_, err := e.ID(core.PK{t.Plantnr, t.Worklineid, t.Specialday}).Update(t)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *Worklinespecialday) SelectOne() (Worklinespecialday, error) {
	e := db.Eloquent.Master()
	var data Worklinespecialday
	_, err := e.ID(core.PK{t.Plantnr, t.Worklineid, t.Specialday}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}
