package log

import (
	"github.com/kataras/iris/v12"
	"leit.com/LAPP_GAAS_GFrame/db"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"leit.com/LAPP_GAAS_GFrame/utils"
	"leit.com/LAPP_GAAS_GFrame/web/middleware/jwts"
	"leit.com/LAPP_GAAS_GFrame/web/models"
	dal "leit.com/LAPP_GAAS_GFrame/web/models/log"
	"leit.com/LAPP_GAAS_GFrame/web/supports"
)

func SelectPackOrderDefault(user *models.Usertab, ctx iris.Context) {
	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfPackOrder, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectPackOrder(predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func SelectPackOrderAndPaging(user *models.Usertab, ctx iris.Context) {
	var pageSize = grmi.GetPageSize(ctx)
	var pageIndex = grmi.GetPageIndex(ctx)

	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfPackOrder, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectPackOrderAndPaging(pageSize, pageIndex, predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.PageOk(ctx, result)
}

var selectMethodMappingOfPackOrder = map[string]func(*models.Usertab, iris.Context){
	"p":      SelectPackOrderAndPaging,
	"paging": SelectPackOrderAndPaging,
}

func SelectPackOrder(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/log/LOG_PackOrder.rest.go", "SelectPackOrder", "查询PackOrder")

	if ctx.URLParamExists("_t") {
		method := ctx.URLParam("_t")
		if selectMethod, ok := selectMethodMappingOfPackOrder[method]; ok {
			selectMethod(user, ctx)
		} else {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		}
	} else {
		SelectPackOrderDefault(user, ctx)
	}
}

func SelectOnePackOrder(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/log/LOG_PackOrder.rest.go", "SelectOnePackOrder", "查询一个PackOrder")

	packOrderId := ctx.Params().GetString("packOrderId")

	engine := db.Eloquent.Master()

	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectOnePackOrder(packOrderId)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	if result == nil {
		supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
