package log

import (
	"github.com/kataras/iris/v12"
	"leit.com/LAPP_GAAS_GFrame/db"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"leit.com/LAPP_GAAS_GFrame/utils"
	"leit.com/LAPP_GAAS_GFrame/web/middleware/jwts"
	"leit.com/LAPP_GAAS_GFrame/web/models"
	dal "leit.com/LAPP_GAAS_GFrame/web/models/log"
	"leit.com/LAPP_GAAS_GFrame/web/supports"
)

func SelectPackOrderItemlstDefault(user *models.Usertab, ctx iris.Context) {
	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfPackOrderItemlst, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectPackOrderItemlst(predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func SelectPackOrderItemlstAndPaging(user *models.Usertab, ctx iris.Context) {
	var pageSize = grmi.GetPageSize(ctx)
	var pageIndex = grmi.GetPageIndex(ctx)

	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfPackOrderItemlst, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectPackOrderItemlstAndPaging(pageSize, pageIndex, predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.PageOk(ctx, result)
}

var selectMethodMappingOfPackOrderItemlst = map[string]func(*models.Usertab, iris.Context){
	"p":      SelectPackOrderItemlstAndPaging,
	"paging": SelectPackOrderItemlstAndPaging,
}

func SelectPackOrderItemlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/log/LOG_PackOrderItemlst.rest.go", "SelectPackOrderItemlst", "查询PackOrderItemlst")

	if ctx.URLParamExists("_t") {
		method := ctx.URLParam("_t")
		if selectMethod, ok := selectMethodMappingOfPackOrderItemlst[method]; ok {
			selectMethod(user, ctx)
		} else {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		}
	} else {
		SelectPackOrderItemlstDefault(user, ctx)
	}
}

func SelectOnePackOrderItemlst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/log/LOG_PackOrderItemlst.rest.go", "SelectOnePackOrderItemlst", "查询一个PackOrderItemlst")

	packOrderId := ctx.Params().GetString("packOrderId")
	pos, err := ctx.Params().GetInt("pos")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}

	engine := db.Eloquent.Master()

	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectOnePackOrderItemlst(packOrderId, pos)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	if result == nil {
		supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
