package qm

import (
	"github.com/kataras/iris/v12"
	"leit.com/LAPP_GAAS_GFrame/db"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"leit.com/LAPP_GAAS_GFrame/utils"
	"leit.com/LAPP_GAAS_GFrame/web/middleware/jwts"
	"leit.com/LAPP_GAAS_GFrame/web/models"
	dal "leit.com/LAPP_GAAS_GFrame/web/models/qm"
	"leit.com/LAPP_GAAS_GFrame/web/supports"
)

func SelectArtViewDefault(user *models.Usertab, ctx iris.Context) {
	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfArtView, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectArtView(predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func SelectArtViewAndPaging(user *models.Usertab, ctx iris.Context) {
	var pageSize = grmi.GetPageSize(ctx)
	var pageIndex = grmi.GetPageIndex(ctx)

	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfArtView, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectArtViewAndPaging(pageSize, pageIndex, predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.PageOk(ctx, result)
}

var selectMethodMappingOfArtView = map[string]func(*models.Usertab, iris.Context){
	"p":      SelectArtViewAndPaging,
	"paging": SelectArtViewAndPaging,
}

func SelectArtView(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/qm/QM_ArtView.rest.go", "SelectArtView", "查询ArtView")

	if ctx.URLParamExists("_t") {
		method := ctx.URLParam("_t")
		if selectMethod, ok := selectMethodMappingOfArtView[method]; ok {
			selectMethod(user, ctx)
		} else {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		}
	} else {
		SelectArtViewDefault(user, ctx)
	}
}

func SelectOneArtView(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/qm/QM_ArtView.rest.go", "SelectOneArtView", "查询一个ArtView")

	artId := ctx.Params().GetString("artId")
	viewId := ctx.Params().GetString("viewId")

	engine := db.Eloquent.Master()

	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectOneArtView(artId, viewId)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	if result == nil {
		supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
