package qm

import (
	"github.com/kataras/iris/v12"
	"leit.com/LAPP_GAAS_GFrame/db"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"leit.com/LAPP_GAAS_GFrame/utils"
	"leit.com/LAPP_GAAS_GFrame/web/middleware/jwts"
	"leit.com/LAPP_GAAS_GFrame/web/models"
	dal "leit.com/LAPP_GAAS_GFrame/web/models/qm"
	"leit.com/LAPP_GAAS_GFrame/web/supports"
)

func SelectViewDefectGrplstDefault(user *models.Usertab, ctx iris.Context) {
	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfViewDefectGrplst, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectViewDefectGrplst(predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func SelectViewDefectGrplstAndPaging(user *models.Usertab, ctx iris.Context) {
	var pageSize = grmi.GetPageSize(ctx)
	var pageIndex = grmi.GetPageIndex(ctx)

	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfViewDefectGrplst, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectViewDefectGrplstAndPaging(pageSize, pageIndex, predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.PageOk(ctx, result)
}

var selectMethodMappingOfViewDefectGrplst = map[string]func(*models.Usertab, iris.Context){
	"p":      SelectViewDefectGrplstAndPaging,
	"paging": SelectViewDefectGrplstAndPaging,
}

func SelectViewDefectGrplst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/qm/QM_ViewDefectGrplst.rest.go", "SelectViewDefectGrplst", "查询ViewDefectGrplst")

	if ctx.URLParamExists("_t") {
		method := ctx.URLParam("_t")
		if selectMethod, ok := selectMethodMappingOfViewDefectGrplst[method]; ok {
			selectMethod(user, ctx)
		} else {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		}
	} else {
		SelectViewDefectGrplstDefault(user, ctx)
	}
}

func SelectOneViewDefectGrplst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/qm/QM_ViewDefectGrplst.rest.go", "SelectOneViewDefectGrplst", "查询一个ViewDefectGrplst")

	viewId := ctx.Params().GetString("viewId")
	defectGrpId := ctx.Params().GetString("defectGrpId")

	engine := db.Eloquent.Master()

	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectOneViewDefectGrplst(viewId, defectGrpId)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	if result == nil {
		supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
