package qm

import (
	"github.com/kataras/iris/v12"
	"leit.com/LAPP_GAAS_GFrame/db"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"leit.com/LAPP_GAAS_GFrame/utils"
	"leit.com/LAPP_GAAS_GFrame/web/middleware/jwts"
	"leit.com/LAPP_GAAS_GFrame/web/models"
	dal "leit.com/LAPP_GAAS_GFrame/web/models/qm"
	"leit.com/LAPP_GAAS_GFrame/web/supports"
)

func SelectViewValstDefault(user *models.Usertab, ctx iris.Context) {
	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfViewValst, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectViewValst(predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func SelectViewValstAndPaging(user *models.Usertab, ctx iris.Context) {
	var pageSize = grmi.GetPageSize(ctx)
	var pageIndex = grmi.GetPageIndex(ctx)

	engine := db.Eloquent.Master()
	predicates, err := grmi.BuildCondtion(dal.MetaOfViewValst, ctx)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectViewValstAndPaging(pageSize, pageIndex, predicates)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.PageOk(ctx, result)
}

var selectMethodMappingOfViewValst = map[string]func(*models.Usertab, iris.Context){
	"p":      SelectViewValstAndPaging,
	"paging": SelectViewValstAndPaging,
}

func SelectViewValst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/qm/QM_ViewValst.rest.go", "SelectViewValst", "查询ViewValst")

	if ctx.URLParamExists("_t") {
		method := ctx.URLParam("_t")
		if selectMethod, ok := selectMethodMappingOfViewValst[method]; ok {
			selectMethod(user, ctx)
		} else {
			supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		}
	} else {
		SelectViewValstDefault(user, ctx)
	}
}

func SelectOneViewValst(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	grmi.Log(user, "/controllers/qm/QM_ViewValst.rest.go", "SelectOneViewValst", "查询一个ViewValst")

	viewId := ctx.Params().GetString("viewId")
	pos, err := ctx.Params().GetInt("pos")
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}

	engine := db.Eloquent.Master()

	dao := dal.NewDAO(engine.NewSession(), user.Pid)
	result, err := dao.SelectOneViewValst(viewId, pos)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	if result == nil {
		supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
