package common

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//Article的实体映射
type Article struct {
	session               *xorm.Session `xorm:"-"`
	PlantNr               int           `xorm:"pk int 'PlantNr'" json:"Article-PlantNr"`
	ArtId                 string        `xorm:"pk nvarchar(40) 'ArtId'" json:"Article-ArtId"`
	Descr1                string        `xorm:"nvarchar(100) 'Descr1' not null" json:"Article-Descr1"`
	Descr2                string        `xorm:"nvarchar(100) 'Descr2' not null" json:"Article-Descr2"`
	ArtTypeId             string        `xorm:"nvarchar(40) 'ArtTypeId' not null" json:"Article-ArtTypeId"`
	ProjectId             string        `xorm:"nvarchar(40) 'ProjectId' not null" json:"Article-ProjectId"`
	CustMaterialId        string        `xorm:"nvarchar(40) 'CustMaterialId' not null" json:"Article-CustMaterialId"`
	AlternativeMaterialId string        `xorm:"nvarchar(40) 'AlternativeMaterialId' not null" json:"Article-AlternativeMaterialId"`
	UomId                 string        `xorm:"nvarchar(40) 'UomId' not null" json:"Article-UomId"`
	QuantityPerHour       float64       `xorm:"float 'QuantityPerHour' not null" json:"Article-QuantityPerHour"`
	DeliveryBuffer        float64       `xorm:"float 'DeliveryBuffer'" json:"Article-DeliveryBuffer"`
	Icon                  string        `xorm:"nvarchar(100) 'Icon' not null" json:"Article-Icon"`
	DisplayColor          string        `xorm:"nvarchar(20) 'DisplayColor' not null" json:"Article-DisplayColor"`
	Document              string        `xorm:"nvarchar(100) 'Document' not null" json:"Article-Document"`
	DefaultRoute          string        `xorm:"nvarchar(40) 'DefaultRoute' not null" json:"Article-DefaultRoute"`
	LotSize               float64       `xorm:"float 'LotSize'" json:"Article-LotSize"`
	ABCClass              string        `xorm:"nvarchar(1) 'ABCClass' not null" json:"Article-ABCClass"`
	ArtSize1              string        `xorm:"nvarchar(100) 'ArtSize1' not null" json:"Article-ArtSize1"`
	ArtSize2              string        `xorm:"nvarchar(100) 'ArtSize2' not null" json:"Article-ArtSize2"`
	ArtSize3              string        `xorm:"nvarchar(100) 'ArtSize3' not null" json:"Article-ArtSize3"`
	ArtSize4              string        `xorm:"nvarchar(100) 'ArtSize4' not null" json:"Article-ArtSize4"`
	ArtSize5              string        `xorm:"nvarchar(100) 'ArtSize5' not null" json:"Article-ArtSize5"`
	ArtSpec1              string        `xorm:"nvarchar(100) 'ArtSpec1' not null" json:"Article-ArtSpec1"`
	ArtSpec2              string        `xorm:"nvarchar(100) 'ArtSpec2' not null" json:"Article-ArtSpec2"`
	ArtSpec3              string        `xorm:"nvarchar(100) 'ArtSpec3' not null" json:"Article-ArtSpec3"`
	ArtSpec4              string        `xorm:"nvarchar(100) 'ArtSpec4' not null" json:"Article-ArtSpec4"`
	ArtSpec5              string        `xorm:"nvarchar(100) 'ArtSpec5' not null" json:"Article-ArtSpec5"`
	DefaultInspectPlanId  string        `xorm:"nvarchar(40) 'DefaultInspectPlanId' not null" json:"Article-DefaultInspectPlanId"`
	LastModify            grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"Article-LastModify"`
	LastUser              string        `xorm:"nvarchar(20) 'LastUser' not null" json:"Article-LastUser"`
	CreateTime            grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"Article-CreateTime"`
}

var MetaOfArticle = map[string]grmi.Field{
	"artid":                 {"ArtId", "string"},
	"descr1":                {"Descr1", "string"},
	"descr2":                {"Descr2", "string"},
	"arttypeid":             {"ArtTypeId", "string"},
	"projectid":             {"ProjectId", "string"},
	"custmaterialid":        {"CustMaterialId", "string"},
	"alternativematerialid": {"AlternativeMaterialId", "string"},
	"uomid":                 {"UomId", "string"},
	"quantityperhour":       {"QuantityPerHour", "float64"},
	"deliverybuffer":        {"DeliveryBuffer", "float64"},
	"icon":                  {"Icon", "string"},
	"displaycolor":          {"DisplayColor", "string"},
	"document":              {"Document", "string"},
	"defaultroute":          {"DefaultRoute", "string"},
	"lotsize":               {"LotSize", "float64"},
	"abcclass":              {"ABCClass", "string"},
	"artsize1":              {"ArtSize1", "string"},
	"artsize2":              {"ArtSize2", "string"},
	"artsize3":              {"ArtSize3", "string"},
	"artsize4":              {"ArtSize4", "string"},
	"artsize5":              {"ArtSize5", "string"},
	"artspec1":              {"ArtSpec1", "string"},
	"artspec2":              {"ArtSpec2", "string"},
	"artspec3":              {"ArtSpec3", "string"},
	"artspec4":              {"ArtSpec4", "string"},
	"artspec5":              {"ArtSpec5", "string"},
	"defaultinspectplanid":  {"DefaultInspectPlanId", "string"},
	"lastmodify":            {"LastModify", "grmi.DateTime"},
	"lastuser":              {"LastUser", "string"},
	"createtime":            {"CreateTime", "grmi.DateTime"},
}

//将Article附加到xorm会话上
func (self *Article) Attach(session *xorm.Session) *Article {
	self.session = session
	return self
}

//插入Article
func (self *Article) Insert() error {
	_, err := self.session.Table("Article").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改Article
func (self *Article) Update() error {
	_, err := self.session.Table("Article").Cols("Descr1", "Descr2", "ArtTypeId", "ProjectId", "CustMaterialId", "AlternativeMaterialId", "UomId", "QuantityPerHour", "DeliveryBuffer", "Icon", "DisplayColor", "Document", "DefaultRoute", "LotSize", "ABCClass", "ArtSize1", "ArtSize2", "ArtSize3", "ArtSize4", "ArtSize5", "ArtSpec1", "ArtSpec2", "ArtSpec3", "ArtSpec4", "ArtSpec5", "DefaultInspectPlanId", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除Article
func (self *Article) Delete() error {
	_, err := self.session.Table("Article").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//Article的表名
var Table_Article = "Article"

//Article.ArtId的列名
var Column_Article_ArtId = "ArtId"

//Article.Descr1的列名
var Column_Article_Descr1 = "Descr1"

//Article.Descr2的列名
var Column_Article_Descr2 = "Descr2"

//Article.ArtTypeId的列名
var Column_Article_ArtTypeId = "ArtTypeId"

//Article.ProjectId的列名
var Column_Article_ProjectId = "ProjectId"

//Article.CustMaterialId的列名
var Column_Article_CustMaterialId = "CustMaterialId"

//Article.AlternativeMaterialId的列名
var Column_Article_AlternativeMaterialId = "AlternativeMaterialId"

//Article.UomId的列名
var Column_Article_UomId = "UomId"

//Article.QuantityPerHour的列名
var Column_Article_QuantityPerHour = "QuantityPerHour"

//Article.DeliveryBuffer的列名
var Column_Article_DeliveryBuffer = "DeliveryBuffer"

//Article.Icon的列名
var Column_Article_Icon = "Icon"

//Article.DisplayColor的列名
var Column_Article_DisplayColor = "DisplayColor"

//Article.Document的列名
var Column_Article_Document = "Document"

//Article.DefaultRoute的列名
var Column_Article_DefaultRoute = "DefaultRoute"

//Article.LotSize的列名
var Column_Article_LotSize = "LotSize"

//Article.ABCClass的列名
var Column_Article_ABCClass = "ABCClass"

//Article.ArtSize1的列名
var Column_Article_ArtSize1 = "ArtSize1"

//Article.ArtSize2的列名
var Column_Article_ArtSize2 = "ArtSize2"

//Article.ArtSize3的列名
var Column_Article_ArtSize3 = "ArtSize3"

//Article.ArtSize4的列名
var Column_Article_ArtSize4 = "ArtSize4"

//Article.ArtSize5的列名
var Column_Article_ArtSize5 = "ArtSize5"

//Article.ArtSpec1的列名
var Column_Article_ArtSpec1 = "ArtSpec1"

//Article.ArtSpec2的列名
var Column_Article_ArtSpec2 = "ArtSpec2"

//Article.ArtSpec3的列名
var Column_Article_ArtSpec3 = "ArtSpec3"

//Article.ArtSpec4的列名
var Column_Article_ArtSpec4 = "ArtSpec4"

//Article.ArtSpec5的列名
var Column_Article_ArtSpec5 = "ArtSpec5"

//Article.DefaultInspectPlanId的列名
var Column_Article_DefaultInspectPlanId = "DefaultInspectPlanId"

//Article的指示器
var IndicatorOfArticle = &Article{}

//创建Article的游标
func BuildArticleCursor() *Article {
	return &Article{}
}

//查找指定键的Article
func (self *DAO) SelectOneArticle(artId string) (*Article, error) {
	var data Article
	ok, err := self.session.Table("Article").ID(core.PK{self.plantNr, artId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的Article
func (self *DAO) DeleteOneArticle(artId string) error {
	_, err := self.session.Table("Article").ID(core.PK{self.plantNr, artId}).Delete(IndicatorOfArticle)
	if err != nil {
		return err
	}
	return nil
}

//批量检查Article
func (self *DAO) checkArticleRecords(records *[]Article) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条Article
func (self *DAO) InsertArticle(records *[]Article) error {
	err := self.checkArticleRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("Article").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条Article
func (self *DAO) UpdateArticle(records *[]Article) error {
	err := self.checkArticleRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("Article").Cols("Descr1", "Descr2", "ArtTypeId", "ProjectId", "CustMaterialId", "AlternativeMaterialId", "UomId", "QuantityPerHour", "DeliveryBuffer", "Icon", "DisplayColor", "Document", "DefaultRoute", "LotSize", "ABCClass", "ArtSize1", "ArtSize2", "ArtSize3", "ArtSize4", "ArtSize5", "ArtSpec1", "ArtSpec2", "ArtSpec3", "ArtSpec4", "ArtSpec5", "DefaultInspectPlanId", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条Article
func (self *DAO) DeleteArticle(records *[]Article) error {
	err := self.checkArticleRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("Article").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询Article
func (self *DAO) SelectArticle(predicates []grmi.Predicate) ([]Article, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]Article, 0)
	err := self.session.Table("Article").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("ArtId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询Article并分页
func (self *DAO) SelectArticleAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]Article, 0)
	err := self.session.Table("Article").SQL("select * from Article where PlantNr = ?"+where+" order by PlantNr, ArtId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("Article").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfArticle)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
