package common

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//CodeParser的实体映射
type CodeParser struct {
	session    *xorm.Session `xorm:"-"`
	PlantNr    int           `xorm:"pk int 'PlantNr'" json:"CodeParser-PlantNr"`
	ParserId   string        `xorm:"pk nvarchar(20) 'ParserId'" json:"CodeParser-ParserId"`
	StartPos   int           `xorm:"int 'StartPos' not null" json:"CodeParser-StartPos"`
	Length     int           `xorm:"int 'Length' not null" json:"CodeParser-Length"`
	ParserType string        `xorm:"nvarchar(20) 'ParserType' not null" json:"CodeParser-ParserType"`
	Descr      string        `xorm:"nvarchar(100) 'Descr' not null" json:"CodeParser-Descr"`
	LastModify grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"CodeParser-LastModify"`
	LastUser   string        `xorm:"nvarchar(20) 'LastUser' not null" json:"CodeParser-LastUser"`
	CreateTime grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"CodeParser-CreateTime"`
}

var MetaOfCodeParser = map[string]grmi.Field{
	"parserid":   {"ParserId", "string"},
	"startpos":   {"StartPos", "int"},
	"length":     {"Length", "int"},
	"parsertype": {"ParserType", "string"},
	"descr":      {"Descr", "string"},
	"lastmodify": {"LastModify", "grmi.DateTime"},
	"lastuser":   {"LastUser", "string"},
	"createtime": {"CreateTime", "grmi.DateTime"},
}

//将CodeParser附加到xorm会话上
func (self *CodeParser) Attach(session *xorm.Session) *CodeParser {
	self.session = session
	return self
}

//插入CodeParser
func (self *CodeParser) Insert() error {
	_, err := self.session.Table("CodeParser").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改CodeParser
func (self *CodeParser) Update() error {
	_, err := self.session.Table("CodeParser").Cols("StartPos", "Length", "ParserType", "Descr", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除CodeParser
func (self *CodeParser) Delete() error {
	_, err := self.session.Table("CodeParser").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//CodeParser的表名
var Table_CodeParser = "CodeParser"

//CodeParser.ParserId的列名
var Column_CodeParser_ParserId = "ParserId"

//CodeParser.StartPos的列名
var Column_CodeParser_StartPos = "StartPos"

//CodeParser.Length的列名
var Column_CodeParser_Length = "Length"

//CodeParser.ParserType的列名
var Column_CodeParser_ParserType = "ParserType"

//CodeParser.Descr的列名
var Column_CodeParser_Descr = "Descr"

//CodeParser的指示器
var IndicatorOfCodeParser = &CodeParser{}

//创建CodeParser的游标
func BuildCodeParserCursor() *CodeParser {
	return &CodeParser{}
}

//查找指定键的CodeParser
func (self *DAO) SelectOneCodeParser(parserId string) (*CodeParser, error) {
	var data CodeParser
	ok, err := self.session.Table("CodeParser").ID(core.PK{self.plantNr, parserId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的CodeParser
func (self *DAO) DeleteOneCodeParser(parserId string) error {
	_, err := self.session.Table("CodeParser").ID(core.PK{self.plantNr, parserId}).Delete(IndicatorOfCodeParser)
	if err != nil {
		return err
	}
	return nil
}

//批量检查CodeParser
func (self *DAO) checkCodeParserRecords(records *[]CodeParser) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条CodeParser
func (self *DAO) InsertCodeParser(records *[]CodeParser) error {
	err := self.checkCodeParserRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("CodeParser").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条CodeParser
func (self *DAO) UpdateCodeParser(records *[]CodeParser) error {
	err := self.checkCodeParserRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("CodeParser").Cols("StartPos", "Length", "ParserType", "Descr", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条CodeParser
func (self *DAO) DeleteCodeParser(records *[]CodeParser) error {
	err := self.checkCodeParserRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("CodeParser").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询CodeParser
func (self *DAO) SelectCodeParser(predicates []grmi.Predicate) ([]CodeParser, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]CodeParser, 0)
	err := self.session.Table("CodeParser").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("ParserId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询CodeParser并分页
func (self *DAO) SelectCodeParserAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]CodeParser, 0)
	err := self.session.Table("CodeParser").SQL("select * from CodeParser where PlantNr = ?"+where+" order by PlantNr, ParserId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("CodeParser").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfCodeParser)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
