package log

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//LOG_PackOrder的实体映射
type PackOrder struct {
	session         *xorm.Session `xorm:"-"`
	PlantNr         int           `xorm:"pk int 'PlantNr'" json:"LOG_PackOrder-PlantNr"`
	PackOrderId     string        `xorm:"pk nvarchar(40) 'PackOrderId'" json:"LOG_PackOrder-PackOrderId"`
	OrderInfo       string        `xorm:"nvarchar(100) 'OrderInfo' not null" json:"LOG_PackOrder-OrderInfo"`
	PackTemplateId  string        `xorm:"nvarchar(40) 'PackTemplateId' not null" json:"LOG_PackOrder-PackTemplateId"`
	PackTypeId      string        `xorm:"nvarchar(40) 'PackTypeId' not null" json:"LOG_PackOrder-PackTypeId"`
	TerminalId      string        `xorm:"nvarchar(40) 'TerminalId' not null" json:"LOG_PackOrder-TerminalId"`
	WorkplaceId     string        `xorm:"nvarchar(20) 'WorkplaceId' not null" json:"LOG_PackOrder-WorkplaceId"`
	PersonNr        int           `xorm:"int 'PersonNr' not null" json:"LOG_PackOrder-PersonNr"`
	ShiftId         string        `xorm:"nvarchar(12) 'ShiftId' not null" json:"LOG_PackOrder-ShiftId"`
	MaterialId      string        `xorm:"nvarchar(40) 'MaterialId' not null" json:"LOG_PackOrder-MaterialId"`
	Status          int           `xorm:"int 'Status' not null" json:"LOG_PackOrder-Status"`
	ShortPack       int           `xorm:"int 'ShortPack' not null" json:"LOG_PackOrder-ShortPack"`
	ShortBy         string        `xorm:"nvarchar(20) 'ShortBy' not null" json:"LOG_PackOrder-ShortBy"`
	ShortTime       grmi.DateTime `xorm:"datetime 'ShortTime'" json:"LOG_PackOrder-ShortTime"`
	ShipStatus      int           `xorm:"int 'ShipStatus' not null" json:"LOG_PackOrder-ShipStatus"`
	PlanQty         float64       `xorm:"float 'PlanQty' not null" json:"LOG_PackOrder-PlanQty"`
	ActQty          float64       `xorm:"float 'ActQty' not null" json:"LOG_PackOrder-ActQty"`
	OpenTime        grmi.DateTime `xorm:"datetime 'OpenTime'" json:"LOG_PackOrder-OpenTime"`
	CloseTime       grmi.DateTime `xorm:"datetime 'CloseTime'" json:"LOG_PackOrder-CloseTime"`
	BackFlushStatus int           `xorm:"int 'BackFlushStatus' not null" json:"LOG_PackOrder-BackFlushStatus"`
	BackFlushTime   grmi.DateTime `xorm:"datetime 'BackFlushTime'" json:"LOG_PackOrder-BackFlushTime"`
	BoxSid          string        `xorm:"nvarchar(40) 'BoxSid' not null" json:"LOG_PackOrder-BoxSid"`
	SyncSid         string        `xorm:"nvarchar(40) 'SyncSid' not null" json:"LOG_PackOrder-SyncSid"`
	PKOCustC1       string        `xorm:"nvarchar(40) 'PKO_CustC1' not null" json:"LOG_PackOrder-PKO_CustC1"`
	PKOCustC2       string        `xorm:"nvarchar(40) 'PKO_CustC2' not null" json:"LOG_PackOrder-PKO_CustC2"`
	PKOCustI1       int           `xorm:"int 'PKO_CustI1' not null" json:"LOG_PackOrder-PKO_CustI1"`
	PKOCustI2       int           `xorm:"int 'PKO_CustI2' not null" json:"LOG_PackOrder-PKO_CustI2"`
	LastModify      grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"LOG_PackOrder-LastModify"`
	LastUser        string        `xorm:"nvarchar(20) 'LastUser' not null" json:"LOG_PackOrder-LastUser"`
	CreateTime      grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"LOG_PackOrder-CreateTime"`
}

var MetaOfPackOrder = map[string]grmi.Field{
	"packorderid":     {"PackOrderId", "string"},
	"orderinfo":       {"OrderInfo", "string"},
	"packtemplateid":  {"PackTemplateId", "string"},
	"packtypeid":      {"PackTypeId", "string"},
	"terminalid":      {"TerminalId", "string"},
	"workplaceid":     {"WorkplaceId", "string"},
	"personnr":        {"PersonNr", "int"},
	"shiftid":         {"ShiftId", "string"},
	"materialid":      {"MaterialId", "string"},
	"status":          {"Status", "int"},
	"shortpack":       {"ShortPack", "int"},
	"shortby":         {"ShortBy", "string"},
	"shorttime":       {"ShortTime", "grmi.DateTime"},
	"shipstatus":      {"ShipStatus", "int"},
	"planqty":         {"PlanQty", "float64"},
	"actqty":          {"ActQty", "float64"},
	"opentime":        {"OpenTime", "grmi.DateTime"},
	"closetime":       {"CloseTime", "grmi.DateTime"},
	"backflushstatus": {"BackFlushStatus", "int"},
	"backflushtime":   {"BackFlushTime", "grmi.DateTime"},
	"boxsid":          {"BoxSid", "string"},
	"syncsid":         {"SyncSid", "string"},
	"pkocustc1":       {"PKO_CustC1", "string"},
	"pkocustc2":       {"PKO_CustC2", "string"},
	"pkocusti1":       {"PKO_CustI1", "int"},
	"pkocusti2":       {"PKO_CustI2", "int"},
	"lastmodify":      {"LastModify", "grmi.DateTime"},
	"lastuser":        {"LastUser", "string"},
	"createtime":      {"CreateTime", "grmi.DateTime"},
}

//将PackOrder附加到xorm会话上
func (self *PackOrder) Attach(session *xorm.Session) *PackOrder {
	self.session = session
	return self
}

//插入PackOrder
func (self *PackOrder) Insert() error {
	_, err := self.session.Table("LOG_PackOrder").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改PackOrder
func (self *PackOrder) Update() error {
	_, err := self.session.Table("LOG_PackOrder").Cols("OrderInfo", "PackTemplateId", "PackTypeId", "TerminalId", "WorkplaceId", "PersonNr", "ShiftId", "MaterialId", "Status", "ShortPack", "ShortBy", "ShortTime", "ShipStatus", "PlanQty", "ActQty", "OpenTime", "CloseTime", "BackFlushStatus", "BackFlushTime", "BoxSid", "SyncSid", "PKO_CustC1", "PKO_CustC2", "PKO_CustI1", "PKO_CustI2", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除PackOrder
func (self *PackOrder) Delete() error {
	_, err := self.session.Table("LOG_PackOrder").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//PackOrder的表名
var Table_PackOrder = "LOG_PackOrder"

//PackOrder.PackOrderId的列名
var Column_PackOrder_PackOrderId = "PackOrderId"

//PackOrder.OrderInfo的列名
var Column_PackOrder_OrderInfo = "OrderInfo"

//PackOrder.PackTemplateId的列名
var Column_PackOrder_PackTemplateId = "PackTemplateId"

//PackOrder.PackTypeId的列名
var Column_PackOrder_PackTypeId = "PackTypeId"

//PackOrder.TerminalId的列名
var Column_PackOrder_TerminalId = "TerminalId"

//PackOrder.WorkplaceId的列名
var Column_PackOrder_WorkplaceId = "WorkplaceId"

//PackOrder.PersonNr的列名
var Column_PackOrder_PersonNr = "PersonNr"

//PackOrder.ShiftId的列名
var Column_PackOrder_ShiftId = "ShiftId"

//PackOrder.MaterialId的列名
var Column_PackOrder_MaterialId = "MaterialId"

//PackOrder.Status的列名
var Column_PackOrder_Status = "Status"

//PackOrder.ShortPack的列名
var Column_PackOrder_ShortPack = "ShortPack"

//PackOrder.ShortBy的列名
var Column_PackOrder_ShortBy = "ShortBy"

//PackOrder.ShortTime的列名
var Column_PackOrder_ShortTime = "ShortTime"

//PackOrder.ShipStatus的列名
var Column_PackOrder_ShipStatus = "ShipStatus"

//PackOrder.PlanQty的列名
var Column_PackOrder_PlanQty = "PlanQty"

//PackOrder.ActQty的列名
var Column_PackOrder_ActQty = "ActQty"

//PackOrder.OpenTime的列名
var Column_PackOrder_OpenTime = "OpenTime"

//PackOrder.CloseTime的列名
var Column_PackOrder_CloseTime = "CloseTime"

//PackOrder.BackFlushStatus的列名
var Column_PackOrder_BackFlushStatus = "BackFlushStatus"

//PackOrder.BackFlushTime的列名
var Column_PackOrder_BackFlushTime = "BackFlushTime"

//PackOrder.BoxSid的列名
var Column_PackOrder_BoxSid = "BoxSid"

//PackOrder.SyncSid的列名
var Column_PackOrder_SyncSid = "SyncSid"

//PackOrder.PKOCustC1的列名
var Column_PackOrder_PKOCustC1 = "PKO_CustC1"

//PackOrder.PKOCustC2的列名
var Column_PackOrder_PKOCustC2 = "PKO_CustC2"

//PackOrder.PKOCustI1的列名
var Column_PackOrder_PKOCustI1 = "PKO_CustI1"

//PackOrder.PKOCustI2的列名
var Column_PackOrder_PKOCustI2 = "PKO_CustI2"

//PackOrder的指示器
var IndicatorOfPackOrder = &PackOrder{}

//创建PackOrder的游标
func BuildPackOrderCursor() *PackOrder {
	return &PackOrder{}
}

//查找指定键的PackOrder
func (self *DAO) SelectOnePackOrder(packOrderId string) (*PackOrder, error) {
	var data PackOrder
	ok, err := self.session.Table("LOG_PackOrder").ID(core.PK{self.plantNr, packOrderId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的PackOrder
func (self *DAO) DeleteOnePackOrder(packOrderId string) error {
	_, err := self.session.Table("LOG_PackOrder").ID(core.PK{self.plantNr, packOrderId}).Delete(IndicatorOfPackOrder)
	if err != nil {
		return err
	}
	return nil
}

//批量检查PackOrder
func (self *DAO) checkPackOrderRecords(records *[]PackOrder) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条PackOrder
func (self *DAO) InsertPackOrder(records *[]PackOrder) error {
	err := self.checkPackOrderRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackOrder").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条PackOrder
func (self *DAO) UpdatePackOrder(records *[]PackOrder) error {
	err := self.checkPackOrderRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackOrder").Cols("OrderInfo", "PackTemplateId", "PackTypeId", "TerminalId", "WorkplaceId", "PersonNr", "ShiftId", "MaterialId", "Status", "ShortPack", "ShortBy", "ShortTime", "ShipStatus", "PlanQty", "ActQty", "OpenTime", "CloseTime", "BackFlushStatus", "BackFlushTime", "BoxSid", "SyncSid", "PKO_CustC1", "PKO_CustC2", "PKO_CustI1", "PKO_CustI2", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条PackOrder
func (self *DAO) DeletePackOrder(records *[]PackOrder) error {
	err := self.checkPackOrderRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackOrder").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询PackOrder
func (self *DAO) SelectPackOrder(predicates []grmi.Predicate) ([]PackOrder, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]PackOrder, 0)
	err := self.session.Table("LOG_PackOrder").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("PackOrderId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询PackOrder并分页
func (self *DAO) SelectPackOrderAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]PackOrder, 0)
	err := self.session.Table("LOG_PackOrder").SQL("select * from LOG_PackOrder where PlantNr = ?"+where+" order by PlantNr, PackOrderId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("LOG_PackOrder").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfPackOrder)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
