package log

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//LOG_PackTemplate的实体映射
type PackTemplate struct {
	session        *xorm.Session `xorm:"-"`
	PlantNr        int           `xorm:"pk int 'PlantNr'" json:"LOG_PackTemplate-PlantNr"`
	PackTemplateId string        `xorm:"pk nvarchar(40) 'PackTemplateId'" json:"LOG_PackTemplate-PackTemplateId"`
	Descr          string        `xorm:"nvarchar(100) 'Descr' not null" json:"LOG_PackTemplate-Descr"`
	ActiveToggle   bool          `xorm:"bit 'ActiveToggle' not null" json:"LOG_PackTemplate-ActiveToggle"`
	PackTypeId     string        `xorm:"nvarchar(40) 'PackTypeId' not null" json:"LOG_PackTemplate-PackTypeId"`
	PrinterId      string        `xorm:"nvarchar(40) 'PrinterId' not null" json:"LOG_PackTemplate-PrinterId"`
	TemplateFile   string        `xorm:"nvarchar(40) 'TemplateFile' not null" json:"LOG_PackTemplate-TemplateFile"`
	PrintQty       int           `xorm:"int 'PrintQty' not null" json:"LOG_PackTemplate-PrintQty"`
	PackOrderSn    string        `xorm:"nvarchar(40) 'PackOrderSn' not null" json:"LOG_PackTemplate-PackOrderSn"`
	BoxSn          string        `xorm:"nvarchar(40) 'BoxSn' not null" json:"LOG_PackTemplate-BoxSn"`
	SyncToggle     bool          `xorm:"bit 'SyncToggle' not null" json:"LOG_PackTemplate-SyncToggle"`
	SyncSn         string        `xorm:"nvarchar(40) 'SyncSn' not null" json:"LOG_PackTemplate-SyncSn"`
	TwoDSheet      int           `xorm:"int 'TwoDSheet' not null" json:"LOG_PackTemplate-TwoDSheet"`
	FullFillToggle bool          `xorm:"bit 'FullFillToggle' not null" json:"LOG_PackTemplate-FullFillToggle"`
	MaterialId     string        `xorm:"nvarchar(40) 'MaterialId' not null" json:"LOG_PackTemplate-MaterialId"`
	PlanQty        int           `xorm:"int 'PlanQty' not null" json:"LOG_PackTemplate-PlanQty"`
	PKTMPCustC1    string        `xorm:"nvarchar(40) 'PKTMP_CustC1' not null" json:"LOG_PackTemplate-PKTMP_CustC1"`
	PKTMPCustC2    string        `xorm:"nvarchar(40) 'PKTMP_CustC2' not null" json:"LOG_PackTemplate-PKTMP_CustC2"`
	PKTMPCustI1    int           `xorm:"int 'PKTMP_CustI1' not null" json:"LOG_PackTemplate-PKTMP_CustI1"`
	PKTMPCustI2    int           `xorm:"int 'PKTMP_CustI2' not null" json:"LOG_PackTemplate-PKTMP_CustI2"`
	LastModify     grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"LOG_PackTemplate-LastModify"`
	LastUser       string        `xorm:"nvarchar(20) 'LastUser' not null" json:"LOG_PackTemplate-LastUser"`
	CreateTime     grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"LOG_PackTemplate-CreateTime"`
}

var MetaOfPackTemplate = map[string]grmi.Field{
	"packtemplateid": {"PackTemplateId", "string"},
	"descr":          {"Descr", "string"},
	"activetoggle":   {"ActiveToggle", "bool"},
	"packtypeid":     {"PackTypeId", "string"},
	"printerid":      {"PrinterId", "string"},
	"templatefile":   {"TemplateFile", "string"},
	"printqty":       {"PrintQty", "int"},
	"packordersn":    {"PackOrderSn", "string"},
	"boxsn":          {"BoxSn", "string"},
	"synctoggle":     {"SyncToggle", "bool"},
	"syncsn":         {"SyncSn", "string"},
	"twodsheet":      {"TwoDSheet", "int"},
	"fullfilltoggle": {"FullFillToggle", "bool"},
	"materialid":     {"MaterialId", "string"},
	"planqty":        {"PlanQty", "int"},
	"pktmpcustc1":    {"PKTMP_CustC1", "string"},
	"pktmpcustc2":    {"PKTMP_CustC2", "string"},
	"pktmpcusti1":    {"PKTMP_CustI1", "int"},
	"pktmpcusti2":    {"PKTMP_CustI2", "int"},
	"lastmodify":     {"LastModify", "grmi.DateTime"},
	"lastuser":       {"LastUser", "string"},
	"createtime":     {"CreateTime", "grmi.DateTime"},
}

//将PackTemplate附加到xorm会话上
func (self *PackTemplate) Attach(session *xorm.Session) *PackTemplate {
	self.session = session
	return self
}

//插入PackTemplate
func (self *PackTemplate) Insert() error {
	_, err := self.session.Table("LOG_PackTemplate").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改PackTemplate
func (self *PackTemplate) Update() error {
	_, err := self.session.Table("LOG_PackTemplate").Cols("Descr", "ActiveToggle", "PackTypeId", "PrinterId", "TemplateFile", "PrintQty", "PackOrderSn", "BoxSn", "SyncToggle", "SyncSn", "TwoDSheet", "FullFillToggle", "MaterialId", "PlanQty", "PKTMP_CustC1", "PKTMP_CustC2", "PKTMP_CustI1", "PKTMP_CustI2", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除PackTemplate
func (self *PackTemplate) Delete() error {
	_, err := self.session.Table("LOG_PackTemplate").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//PackTemplate的表名
var Table_PackTemplate = "LOG_PackTemplate"

//PackTemplate.PackTemplateId的列名
var Column_PackTemplate_PackTemplateId = "PackTemplateId"

//PackTemplate.Descr的列名
var Column_PackTemplate_Descr = "Descr"

//PackTemplate.ActiveToggle的列名
var Column_PackTemplate_ActiveToggle = "ActiveToggle"

//PackTemplate.PackTypeId的列名
var Column_PackTemplate_PackTypeId = "PackTypeId"

//PackTemplate.PrinterId的列名
var Column_PackTemplate_PrinterId = "PrinterId"

//PackTemplate.TemplateFile的列名
var Column_PackTemplate_TemplateFile = "TemplateFile"

//PackTemplate.PrintQty的列名
var Column_PackTemplate_PrintQty = "PrintQty"

//PackTemplate.PackOrderSn的列名
var Column_PackTemplate_PackOrderSn = "PackOrderSn"

//PackTemplate.BoxSn的列名
var Column_PackTemplate_BoxSn = "BoxSn"

//PackTemplate.SyncToggle的列名
var Column_PackTemplate_SyncToggle = "SyncToggle"

//PackTemplate.SyncSn的列名
var Column_PackTemplate_SyncSn = "SyncSn"

//PackTemplate.TwoDSheet的列名
var Column_PackTemplate_TwoDSheet = "TwoDSheet"

//PackTemplate.FullFillToggle的列名
var Column_PackTemplate_FullFillToggle = "FullFillToggle"

//PackTemplate.MaterialId的列名
var Column_PackTemplate_MaterialId = "MaterialId"

//PackTemplate.PlanQty的列名
var Column_PackTemplate_PlanQty = "PlanQty"

//PackTemplate.PKTMPCustC1的列名
var Column_PackTemplate_PKTMPCustC1 = "PKTMP_CustC1"

//PackTemplate.PKTMPCustC2的列名
var Column_PackTemplate_PKTMPCustC2 = "PKTMP_CustC2"

//PackTemplate.PKTMPCustI1的列名
var Column_PackTemplate_PKTMPCustI1 = "PKTMP_CustI1"

//PackTemplate.PKTMPCustI2的列名
var Column_PackTemplate_PKTMPCustI2 = "PKTMP_CustI2"

//PackTemplate的指示器
var IndicatorOfPackTemplate = &PackTemplate{}

//创建PackTemplate的游标
func BuildPackTemplateCursor() *PackTemplate {
	return &PackTemplate{}
}

//查找指定键的PackTemplate
func (self *DAO) SelectOnePackTemplate(packTemplateId string) (*PackTemplate, error) {
	var data PackTemplate
	ok, err := self.session.Table("LOG_PackTemplate").ID(core.PK{self.plantNr, packTemplateId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的PackTemplate
func (self *DAO) DeleteOnePackTemplate(packTemplateId string) error {
	_, err := self.session.Table("LOG_PackTemplate").ID(core.PK{self.plantNr, packTemplateId}).Delete(IndicatorOfPackTemplate)
	if err != nil {
		return err
	}
	return nil
}

//批量检查PackTemplate
func (self *DAO) checkPackTemplateRecords(records *[]PackTemplate) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条PackTemplate
func (self *DAO) InsertPackTemplate(records *[]PackTemplate) error {
	err := self.checkPackTemplateRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackTemplate").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条PackTemplate
func (self *DAO) UpdatePackTemplate(records *[]PackTemplate) error {
	err := self.checkPackTemplateRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackTemplate").Cols("Descr", "ActiveToggle", "PackTypeId", "PrinterId", "TemplateFile", "PrintQty", "PackOrderSn", "BoxSn", "SyncToggle", "SyncSn", "TwoDSheet", "FullFillToggle", "MaterialId", "PlanQty", "PKTMP_CustC1", "PKTMP_CustC2", "PKTMP_CustI1", "PKTMP_CustI2", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条PackTemplate
func (self *DAO) DeletePackTemplate(records *[]PackTemplate) error {
	err := self.checkPackTemplateRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackTemplate").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询PackTemplate
func (self *DAO) SelectPackTemplate(predicates []grmi.Predicate) ([]PackTemplate, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]PackTemplate, 0)
	err := self.session.Table("LOG_PackTemplate").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("PackTemplateId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询PackTemplate并分页
func (self *DAO) SelectPackTemplateAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]PackTemplate, 0)
	err := self.session.Table("LOG_PackTemplate").SQL("select * from LOG_PackTemplate where PlantNr = ?"+where+" order by PlantNr, PackTemplateId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("LOG_PackTemplate").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfPackTemplate)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
