package log

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//LOG_PackType的实体映射
type PackType struct {
	session          *xorm.Session `xorm:"-"`
	PlantNr          int           `xorm:"pk int 'PlantNr'" json:"LOG_PackType-PlantNr"`
	PackTypeId       string        `xorm:"pk nvarchar(40) 'PackTypeId'" json:"LOG_PackType-PackTypeId"`
	Descr            string        `xorm:"nvarchar(100) 'Descr' not null" json:"LOG_PackType-Descr"`
	SortWay          string        `xorm:"nvarchar(10) 'SortWay' not null" json:"LOG_PackType-SortWay"`
	VerifySlotToggle bool          `xorm:"bit 'VerifySlotToggle' not null" json:"LOG_PackType-VerifySlotToggle"`
	VerifyRule       string        `xorm:"nvarchar(100) 'VerifyRule' not null" json:"LOG_PackType-VerifyRule"`
	Length           float64       `xorm:"float 'Length' not null" json:"LOG_PackType-Length"`
	Width            float64       `xorm:"float 'Width' not null" json:"LOG_PackType-Width"`
	Height           float64       `xorm:"float 'Height' not null" json:"LOG_PackType-Height"`
	SizeUom          string        `xorm:"nvarchar(10) 'SizeUom' not null" json:"LOG_PackType-SizeUom"`
	Weight           float64       `xorm:"float 'Weight' not null" json:"LOG_PackType-Weight"`
	WeightUom        string        `xorm:"nvarchar(10) 'WeightUom' not null" json:"LOG_PackType-WeightUom"`
	LastModify       grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"LOG_PackType-LastModify"`
	LastUser         string        `xorm:"nvarchar(20) 'LastUser' not null" json:"LOG_PackType-LastUser"`
	CreateTime       grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"LOG_PackType-CreateTime"`
}

var MetaOfPackType = map[string]grmi.Field{
	"packtypeid":       {"PackTypeId", "string"},
	"descr":            {"Descr", "string"},
	"sortway":          {"SortWay", "string"},
	"verifyslottoggle": {"VerifySlotToggle", "bool"},
	"verifyrule":       {"VerifyRule", "string"},
	"length":           {"Length", "float64"},
	"width":            {"Width", "float64"},
	"height":           {"Height", "float64"},
	"sizeuom":          {"SizeUom", "string"},
	"weight":           {"Weight", "float64"},
	"weightuom":        {"WeightUom", "string"},
	"lastmodify":       {"LastModify", "grmi.DateTime"},
	"lastuser":         {"LastUser", "string"},
	"createtime":       {"CreateTime", "grmi.DateTime"},
}

//将PackType附加到xorm会话上
func (self *PackType) Attach(session *xorm.Session) *PackType {
	self.session = session
	return self
}

//插入PackType
func (self *PackType) Insert() error {
	_, err := self.session.Table("LOG_PackType").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改PackType
func (self *PackType) Update() error {
	_, err := self.session.Table("LOG_PackType").Cols("Descr", "SortWay", "VerifySlotToggle", "VerifyRule", "Length", "Width", "Height", "SizeUom", "Weight", "WeightUom", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除PackType
func (self *PackType) Delete() error {
	_, err := self.session.Table("LOG_PackType").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//PackType的表名
var Table_PackType = "LOG_PackType"

//PackType.PackTypeId的列名
var Column_PackType_PackTypeId = "PackTypeId"

//PackType.Descr的列名
var Column_PackType_Descr = "Descr"

//PackType.SortWay的列名
var Column_PackType_SortWay = "SortWay"

//PackType.VerifySlotToggle的列名
var Column_PackType_VerifySlotToggle = "VerifySlotToggle"

//PackType.VerifyRule的列名
var Column_PackType_VerifyRule = "VerifyRule"

//PackType.Length的列名
var Column_PackType_Length = "Length"

//PackType.Width的列名
var Column_PackType_Width = "Width"

//PackType.Height的列名
var Column_PackType_Height = "Height"

//PackType.SizeUom的列名
var Column_PackType_SizeUom = "SizeUom"

//PackType.Weight的列名
var Column_PackType_Weight = "Weight"

//PackType.WeightUom的列名
var Column_PackType_WeightUom = "WeightUom"

//PackType的指示器
var IndicatorOfPackType = &PackType{}

//创建PackType的游标
func BuildPackTypeCursor() *PackType {
	return &PackType{}
}

//查找指定键的PackType
func (self *DAO) SelectOnePackType(packTypeId string) (*PackType, error) {
	var data PackType
	ok, err := self.session.Table("LOG_PackType").ID(core.PK{self.plantNr, packTypeId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的PackType
func (self *DAO) DeleteOnePackType(packTypeId string) error {
	_, err := self.session.Table("LOG_PackType").ID(core.PK{self.plantNr, packTypeId}).Delete(IndicatorOfPackType)
	if err != nil {
		return err
	}
	return nil
}

//批量检查PackType
func (self *DAO) checkPackTypeRecords(records *[]PackType) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条PackType
func (self *DAO) InsertPackType(records *[]PackType) error {
	err := self.checkPackTypeRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackType").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条PackType
func (self *DAO) UpdatePackType(records *[]PackType) error {
	err := self.checkPackTypeRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackType").Cols("Descr", "SortWay", "VerifySlotToggle", "VerifyRule", "Length", "Width", "Height", "SizeUom", "Weight", "WeightUom", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条PackType
func (self *DAO) DeletePackType(records *[]PackType) error {
	err := self.checkPackTypeRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("LOG_PackType").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询PackType
func (self *DAO) SelectPackType(predicates []grmi.Predicate) ([]PackType, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]PackType, 0)
	err := self.session.Table("LOG_PackType").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("PackTypeId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询PackType并分页
func (self *DAO) SelectPackTypeAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]PackType, 0)
	err := self.session.Table("LOG_PackType").SQL("select * from LOG_PackType where PlantNr = ?"+where+" order by PlantNr, PackTypeId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("LOG_PackType").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfPackType)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
