package qm

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//QM_ArtView的实体映射
type ArtView struct {
	session     *xorm.Session `xorm:"-"`
	PlantNr     int           `xorm:"pk int 'PlantNr'" json:"QM_ArtView-PlantNr"`
	ArtId       string        `xorm:"pk nvarchar(40) 'ArtId'" json:"QM_ArtView-ArtId"`
	ViewId      string        `xorm:"pk nvarchar(40) 'ViewId'" json:"QM_ArtView-ViewId"`
	Pos         int           `xorm:"int 'Pos' not null" json:"QM_ArtView-Pos"`
	Descr       string        `xorm:"nvarchar(100) 'Descr' not null" json:"QM_ArtView-Descr"`
	ViewPicture string        `xorm:"nvarchar(100) 'ViewPicture' not null" json:"QM_ArtView-ViewPicture"`
	LastModify  grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"QM_ArtView-LastModify"`
	LastUser    string        `xorm:"nvarchar(20) 'LastUser' not null" json:"QM_ArtView-LastUser"`
	CreateTime  grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"QM_ArtView-CreateTime"`
}

var MetaOfArtView = map[string]grmi.Field{
	"artid":       {"ArtId", "string"},
	"viewid":      {"ViewId", "string"},
	"pos":         {"Pos", "int"},
	"descr":       {"Descr", "string"},
	"viewpicture": {"ViewPicture", "string"},
	"lastmodify":  {"LastModify", "grmi.DateTime"},
	"lastuser":    {"LastUser", "string"},
	"createtime":  {"CreateTime", "grmi.DateTime"},
}

//将ArtView附加到xorm会话上
func (self *ArtView) Attach(session *xorm.Session) *ArtView {
	self.session = session
	return self
}

//插入ArtView
func (self *ArtView) Insert() error {
	_, err := self.session.Table("QM_ArtView").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改ArtView
func (self *ArtView) Update() error {
	_, err := self.session.Table("QM_ArtView").Cols("Pos", "Descr", "ViewPicture", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除ArtView
func (self *ArtView) Delete() error {
	_, err := self.session.Table("QM_ArtView").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//ArtView的表名
var Table_ArtView = "QM_ArtView"

//ArtView.ArtId的列名
var Column_ArtView_ArtId = "ArtId"

//ArtView.ViewId的列名
var Column_ArtView_ViewId = "ViewId"

//ArtView.Pos的列名
var Column_ArtView_Pos = "Pos"

//ArtView.Descr的列名
var Column_ArtView_Descr = "Descr"

//ArtView.ViewPicture的列名
var Column_ArtView_ViewPicture = "ViewPicture"

//ArtView的指示器
var IndicatorOfArtView = &ArtView{}

//创建ArtView的游标
func BuildArtViewCursor() *ArtView {
	return &ArtView{}
}

//查找指定键的ArtView
func (self *DAO) SelectOneArtView(artId string, viewId string) (*ArtView, error) {
	var data ArtView
	ok, err := self.session.Table("QM_ArtView").ID(core.PK{self.plantNr, artId, viewId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的ArtView
func (self *DAO) DeleteOneArtView(artId string, viewId string) error {
	_, err := self.session.Table("QM_ArtView").ID(core.PK{self.plantNr, artId, viewId}).Delete(IndicatorOfArtView)
	if err != nil {
		return err
	}
	return nil
}

//批量检查ArtView
func (self *DAO) checkArtViewRecords(records *[]ArtView) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条ArtView
func (self *DAO) InsertArtView(records *[]ArtView) error {
	err := self.checkArtViewRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_ArtView").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条ArtView
func (self *DAO) UpdateArtView(records *[]ArtView) error {
	err := self.checkArtViewRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_ArtView").Cols("Pos", "Descr", "ViewPicture", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条ArtView
func (self *DAO) DeleteArtView(records *[]ArtView) error {
	err := self.checkArtViewRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_ArtView").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询ArtView
func (self *DAO) SelectArtView(predicates []grmi.Predicate) ([]ArtView, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]ArtView, 0)
	err := self.session.Table("QM_ArtView").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("ArtId").OrderBy("ViewId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询ArtView并分页
func (self *DAO) SelectArtViewAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]ArtView, 0)
	err := self.session.Table("QM_ArtView").SQL("select * from QM_ArtView where PlantNr = ?"+where+" order by PlantNr, ArtId, ViewId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("QM_ArtView").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfArtView)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
