package qm

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//QM_View的实体映射
type View struct {
	session     *xorm.Session `xorm:"-"`
	PlantNr     int           `xorm:"pk int 'PlantNr'" json:"QM_View-PlantNr"`
	ViewId      string        `xorm:"pk nvarchar(20) 'ViewId'" json:"QM_View-ViewId"`
	Descr       string        `xorm:"nvarchar(100) 'Descr' not null" json:"QM_View-Descr"`
	Picture     string        `xorm:"nvarchar(1000) 'Picture' not null" json:"QM_View-Picture"`
	Abrev       string        `xorm:"nvarchar(20) 'Abrev' not null" json:"QM_View-Abrev"`
	Width       int           `xorm:"int 'Width' not null" json:"QM_View-Width"`
	Height      int           `xorm:"int 'Height' not null" json:"QM_View-Height"`
	XLineColor  string        `xorm:"nvarchar(20) 'XLineColor' not null" json:"QM_View-XLineColor"`
	YLineColor  string        `xorm:"nvarchar(20) 'YLineColor' not null" json:"QM_View-YLineColor"`
	UseXYToggle bool          `xorm:"bit 'UseXYToggle' not null" json:"QM_View-UseXYToggle"`
	LastModify  grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"QM_View-LastModify"`
	LastUser    string        `xorm:"nvarchar(20) 'LastUser' not null" json:"QM_View-LastUser"`
	CreateTime  grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"QM_View-CreateTime"`
}

var MetaOfView = map[string]grmi.Field{
	"viewid":      {"ViewId", "string"},
	"descr":       {"Descr", "string"},
	"picture":     {"Picture", "string"},
	"abrev":       {"Abrev", "string"},
	"width":       {"Width", "int"},
	"height":      {"Height", "int"},
	"xlinecolor":  {"XLineColor", "string"},
	"ylinecolor":  {"YLineColor", "string"},
	"usexytoggle": {"UseXYToggle", "bool"},
	"lastmodify":  {"LastModify", "grmi.DateTime"},
	"lastuser":    {"LastUser", "string"},
	"createtime":  {"CreateTime", "grmi.DateTime"},
}

//将View附加到xorm会话上
func (self *View) Attach(session *xorm.Session) *View {
	self.session = session
	return self
}

//插入View
func (self *View) Insert() error {
	_, err := self.session.Table("QM_View").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改View
func (self *View) Update() error {
	_, err := self.session.Table("QM_View").Cols("Descr", "Picture", "Abrev", "Width", "Height", "XLineColor", "YLineColor", "UseXYToggle", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除View
func (self *View) Delete() error {
	_, err := self.session.Table("QM_View").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//View的表名
var Table_View = "QM_View"

//View.ViewId的列名
var Column_View_ViewId = "ViewId"

//View.Descr的列名
var Column_View_Descr = "Descr"

//View.Picture的列名
var Column_View_Picture = "Picture"

//View.Abrev的列名
var Column_View_Abrev = "Abrev"

//View.Width的列名
var Column_View_Width = "Width"

//View.Height的列名
var Column_View_Height = "Height"

//View.XLineColor的列名
var Column_View_XLineColor = "XLineColor"

//View.YLineColor的列名
var Column_View_YLineColor = "YLineColor"

//View.UseXYToggle的列名
var Column_View_UseXYToggle = "UseXYToggle"

//View的指示器
var IndicatorOfView = &View{}

//创建View的游标
func BuildViewCursor() *View {
	return &View{}
}

//查找指定键的View
func (self *DAO) SelectOneView(viewId string) (*View, error) {
	var data View
	ok, err := self.session.Table("QM_View").ID(core.PK{self.plantNr, viewId}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的View
func (self *DAO) DeleteOneView(viewId string) error {
	_, err := self.session.Table("QM_View").ID(core.PK{self.plantNr, viewId}).Delete(IndicatorOfView)
	if err != nil {
		return err
	}
	return nil
}

//批量检查View
func (self *DAO) checkViewRecords(records *[]View) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条View
func (self *DAO) InsertView(records *[]View) error {
	err := self.checkViewRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_View").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条View
func (self *DAO) UpdateView(records *[]View) error {
	err := self.checkViewRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_View").Cols("Descr", "Picture", "Abrev", "Width", "Height", "XLineColor", "YLineColor", "UseXYToggle", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条View
func (self *DAO) DeleteView(records *[]View) error {
	err := self.checkViewRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_View").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询View
func (self *DAO) SelectView(predicates []grmi.Predicate) ([]View, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]View, 0)
	err := self.session.Table("QM_View").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("ViewId").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询View并分页
func (self *DAO) SelectViewAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]View, 0)
	err := self.session.Table("QM_View").SQL("select * from QM_View where PlantNr = ?"+where+" order by PlantNr, ViewId offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("QM_View").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfView)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
