package qm

import (
	"errors"
	"github.com/go-xorm/xorm"
	"leit.com/LAPP_GAAS_GFrame/grmi"
	"xorm.io/core"
)

//QM_ViewValst的实体映射
type ViewValst struct {
	session    *xorm.Session `xorm:"-"`
	PlantNr    int           `xorm:"pk int 'PlantNr'" json:"QM_ViewValst-PlantNr"`
	ViewId     string        `xorm:"pk nvarchar(20) 'ViewId'" json:"QM_ViewValst-ViewId"`
	Pos        int           `xorm:"pk int 'Pos'" json:"QM_ViewValst-Pos"`
	ValName    string        `xorm:"nvarchar(16) 'ValName' not null" json:"QM_ViewValst-ValName"`
	XStart     int           `xorm:"int 'XStart' not null" json:"QM_ViewValst-XStart"`
	YStart     int           `xorm:"int 'YStart' not null" json:"QM_ViewValst-YStart"`
	Width      int           `xorm:"int 'Width' not null" json:"QM_ViewValst-Width"`
	Height     int           `xorm:"int 'Height' not null" json:"QM_ViewValst-Height"`
	Descr      string        `xorm:"nvarchar(100) 'Descr' not null" json:"QM_ViewValst-Descr"`
	LastModify grmi.DateTime `xorm:"datetime 'LastModify' not null updated" json:"QM_ViewValst-LastModify"`
	LastUser   string        `xorm:"nvarchar(20) 'LastUser' not null" json:"QM_ViewValst-LastUser"`
	CreateTime grmi.DateTime `xorm:"datetime 'CreateTime' not null created" json:"QM_ViewValst-CreateTime"`
}

var MetaOfViewValst = map[string]grmi.Field{
	"viewid":     {"ViewId", "string"},
	"pos":        {"Pos", "int"},
	"valname":    {"ValName", "string"},
	"xstart":     {"XStart", "int"},
	"ystart":     {"YStart", "int"},
	"width":      {"Width", "int"},
	"height":     {"Height", "int"},
	"descr":      {"Descr", "string"},
	"lastmodify": {"LastModify", "grmi.DateTime"},
	"lastuser":   {"LastUser", "string"},
	"createtime": {"CreateTime", "grmi.DateTime"},
}

//将ViewValst附加到xorm会话上
func (self *ViewValst) Attach(session *xorm.Session) *ViewValst {
	self.session = session
	return self
}

//插入ViewValst
func (self *ViewValst) Insert() error {
	_, err := self.session.Table("QM_ViewValst").Insert(self)
	if err != nil {
		return err
	}
	return nil
}

//修改ViewValst
func (self *ViewValst) Update() error {
	_, err := self.session.Table("QM_ViewValst").Cols("ValName", "XStart", "YStart", "Width", "Height", "Descr", "LastUser").Update(self)
	if err != nil {
		return err
	}
	return nil
}

//删除ViewValst
func (self *ViewValst) Delete() error {
	_, err := self.session.Table("QM_ViewValst").Delete(self)
	if err != nil {
		return err
	}
	return nil
}

//ViewValst的表名
var Table_ViewValst = "QM_ViewValst"

//ViewValst.ViewId的列名
var Column_ViewValst_ViewId = "ViewId"

//ViewValst.Pos的列名
var Column_ViewValst_Pos = "Pos"

//ViewValst.ValName的列名
var Column_ViewValst_ValName = "ValName"

//ViewValst.XStart的列名
var Column_ViewValst_XStart = "XStart"

//ViewValst.YStart的列名
var Column_ViewValst_YStart = "YStart"

//ViewValst.Width的列名
var Column_ViewValst_Width = "Width"

//ViewValst.Height的列名
var Column_ViewValst_Height = "Height"

//ViewValst.Descr的列名
var Column_ViewValst_Descr = "Descr"

//ViewValst的指示器
var IndicatorOfViewValst = &ViewValst{}

//创建ViewValst的游标
func BuildViewValstCursor() *ViewValst {
	return &ViewValst{}
}

//查找指定键的ViewValst
func (self *DAO) SelectOneViewValst(viewId string, pos int) (*ViewValst, error) {
	var data ViewValst
	ok, err := self.session.Table("QM_ViewValst").ID(core.PK{self.plantNr, viewId, pos}).Get(&data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return &data, nil
}

//删除指定键的ViewValst
func (self *DAO) DeleteOneViewValst(viewId string, pos int) error {
	_, err := self.session.Table("QM_ViewValst").ID(core.PK{self.plantNr, viewId, pos}).Delete(IndicatorOfViewValst)
	if err != nil {
		return err
	}
	return nil
}

//批量检查ViewValst
func (self *DAO) checkViewValstRecords(records *[]ViewValst) error {
	for _, record := range *records {
		if self.plantNr != record.PlantNr {
			return errors.New("记录不在范围内!")
		}
	}
	return nil
}

//插入多条ViewValst
func (self *DAO) InsertViewValst(records *[]ViewValst) error {
	err := self.checkViewValstRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_ViewValst").Insert(records)
	if err != nil {
		return err
	}
	return nil
}

//修改多条ViewValst
func (self *DAO) UpdateViewValst(records *[]ViewValst) error {
	err := self.checkViewValstRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_ViewValst").Cols("ValName", "XStart", "YStart", "Width", "Height", "Descr", "LastUser").Update(records)
	if err != nil {
		return err
	}
	return nil
}

//删除多条ViewValst
func (self *DAO) DeleteViewValst(records *[]ViewValst) error {
	err := self.checkViewValstRecords(records)
	if err != nil {
		return err
	}
	_, err = self.session.Table("QM_ViewValst").Delete(records)
	if err != nil {
		return err
	}
	return nil
}

//按条件查询ViewValst
func (self *DAO) SelectViewValst(predicates []grmi.Predicate) ([]ViewValst, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	data := make([]ViewValst, 0)
	err := self.session.Table("QM_ViewValst").Where("PlantNr = ?"+where, parameters...).OrderBy("PlantNr").OrderBy("ViewId").OrderBy("Pos").Find(&data)
	if err != nil {
		return nil, err
	}
	return data, nil
}

//按条件查询ViewValst并分页
func (self *DAO) SelectViewValstAndPaging(pageSize int, pageIndex int, predicates []grmi.Predicate) (grmi.PagingResult, error) {
	parameters := make([]interface{}, 0, 10)
	parameters = append(parameters, self.plantNr)
	where := ""
	if predicates != nil {
		for _, predicate := range predicates {
			where += " and " + predicate.ColumnName + " = ?"
			parameters = append(parameters, predicate.Value)
		}
	}
	parameters = append(parameters, (pageIndex-1)*pageSize, pageSize)
	data := make([]ViewValst, 0)
	err := self.session.Table("QM_ViewValst").SQL("select * from QM_ViewValst where PlantNr = ?"+where+" order by PlantNr, ViewId, Pos offset ? row fetch next ? row only", parameters...).Find(&data)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	count, err := self.session.Table("QM_ViewValst").Where("PlantNr = ? ", self.plantNr).Count(IndicatorOfViewValst)
	if err != nil {
		return grmi.PagingResult{}, err
	}
	return grmi.PagingResult{data, count, pageIndex, pageSize}, nil
}
